program vnl2bmp;
uses
  crt, nibbserv;
const
  maxcolors = 255;
  usedColors = 3;
  bpp = 4; { bits pro pixel }
  xRes = 80;
  yRes = 48;
type
  bmpheader = packed record
    { ----- Fileinfo ----- }
    bfType      : word;      { Dateikennung (="BM") }
    bfSize      : longint;   { Laenge der Datei in Bytes }
    bfReserved1 : word;      { immer NULL }
    bfReserved2 : word;      { immer NULL }
    bfOffBits   : longint;   { Offset der Bilddaten }
    { ----- Bildinfo ----- }
    biSize      : longint;   { Groesse der Bildinfo und Farbtabelle in Bytes }
    biWidth     : longint;   { Breite der Bitmap in Pixeln }
    biHeight    : longint;   { Hoehe der Bitmap in Pixeln }
    biPlanes    : word;      { Anzahl der Ebenen (immer 1) }
    biBitCount  : word;      { Anzahl der Bits pro Pixel (1,2,4,8,24) }
    biCompress  : longint;   { Komprimierungsart (0 => unkomprimiert) }
    biSizeImage : longint;   { Groesse der Bilddaten in Bytes. }
    biXPPM      : longint;   { X-Aufloesung in Pixel pro Meter }
    biYPPM      : longint;   { Y-Aufloesung in Pixel pro Meter }
    biClrUsed   : longint;   { Anzahl der verwendeten Farben der Farbpalette
                              (falls 0 => max. Anzahl verwendet) }
    biClrImport : longint;   { Anzahl der Farben die zur Darstellung als "wichtig"
                              betrachtet werden. (falls 0 => max. Anzahl verwendet) }
  end;
    { ----- Farbtabelle ----- }
    { hier folgt die Farbtabelle, falls vorhanden }
  BMP_RGBQuad = record
                  Blue :Byte; (*Intensitt der Farbe BLAU*)
                  Green:Byte; (*Intensitt der Farbe GRN*)
                  Red  :Byte; (*Intensitt der Farbe ROT*)
                  Reserved:Byte; (* Reservieter Eintrag (mu 0 sein)*)
                end;
  Colors= array[0..maxcolors] of BMP_RGBQuad;
var
  h: bmpHeader;
  bmpColors: colors;
  feld: array[1..80, 1..48] of boolean;
  vordergrund, hintergrund: byte;
  dateiName: string; {ohne Dateinamenserweiterung!}

function EXP(Basis, Exponent: word): longint; (* Ausrechnen von Exponentialfunktionen*)
var
  n : word;
  a : longint;
begin
  a := Basis;
  if Exponent = 0 then a := 1 else
    for n := 2 to Exponent do a := a * Basis;
  EXP := a;
end;

procedure bmpSchreiben;
var
  f: file;
  f2: file of byte;
  i, j, k, x, y: byte;
begin
  assign( f, dateiName + '.bmp' );
  rewrite( f, 1 );
  blockwrite( f, h, sizeof( h ));
{  blockwrite( f, bmpColors, EXP(2, h.biBitCount) * sizeof(BMP_RGBQuad));}
  blockwrite( f, bmpColors, h.biClrUsed * sizeof( BMP_RGBQuad ) );
  close( f );
  assign( f2, dateiName + '.bmp' );
  reset( f2 );
  seek( f2, h.bfOffbits );
  for i := 48 downto 1 do
    for j := 1 to ( 10 * bpp ) do
    begin
      if feld[ ( j * ( 8 div bpp ) - 1 ), i ] then x := 2 else x := 1;
      if feld[ j * ( 8 div bpp ), i ] then y := 2 else y := 1;
      y := y xor ( x shl 4 );
      write( f2, y );
    end;
  close( f2 );
end;

procedure headerInfo;
const
  s = 20;
begin
  clrscr;
  writeln( 'Datei: ', dateiName );
  writeln;
  writeln( ' ----- Fileinfo ----- ' );
  writeln( h.bfType:s, ' Dateikennung (="BM")' );
  writeln( h.bfSize:s, ' Laenge der Datei in Bytes' );
  writeln( h.bfReserved1:s, ' immer NULL' );
  writeln( h.bfReserved2:s, ' immer NULL' );
  writeln( h.bfOffBits:s, ' Offset der Bilddaten' );
  writeln( ' ----- Bildinfo ----- ' );
  writeln( h.biSize:s, ' Groesse der Bildinfo und Farbtabelle in Bytes' );
  writeln( h.biWidth:s, ' Breite der Bitmap in Pixeln' );
  writeln( h.biHeight:s, ' Hoehe der Bitmap in Pixeln' );
  writeln( h.biPlanes:s, ' Anzahl der Ebenen (immer 1)' );
  writeln( h.biBitCount:s, ' Anzahl der Bits pro Pixel (1,2,4,8,24)' );
  writeln( h.biCompress:s, ' Komprimierungsart (0 => unkomprimiert)' );
  writeln( h.biSizeImage:s, ' Groesse der Bilddaten in Bytes.' );
  writeln( h.biXPPM:s, ' X-Aufloesung in Pixel pro Meter' );
  writeln( h.biYPPM:s, ' Y-Aufloesung in Pixel pro Meter' );
  writeln( h.biClrUsed:s, ' Anzahl der verwendeten Farben der Farbpalette' );
  writeln( '                     (falls 0 => max. Anzahl verwendet) ' );
  writeln( h.biClrImport:s, ' Anzahl der Farben die zur Darstellung als "wichtig" ');
  writeln( '                     betrachtet werden. (falls 0 => max. Anzahl verwendet)' );
  writeln;
  writeln( 'Grsse des Headers: ', sizeOf( h ), ' Bytes');
end;

procedure initHeader;
var
  xWidth: word;
begin
  h.bfType := 19778;
  xWidth := xRes;
  while ( xWidth * bpp ) mod ( 8 * 4 ) > 0 do inc( xWidth );
    {Breite in bytes muss durch 4 teilbar sein!}
  h.bfSize := sizeOf( h ) + usedColors * sizeOf( bmpColors[0] ) + ( xWidth * yRes * bpp ) div 8;
    { Grsse: header + farbtabelle + pixel }
  h.bfReserved1 := 0;
  h.bfReserved2 := 0;
  h.bfOffBits := sizeOf( h ) + usedColors * sizeOf( bmpColors[0] );
  h.biSize := 40; {? Scheint immer 40 zu sein. Ist wohl nicht so wichtig...}
  h.biWidth := xRes;
  h.biHeight := yRes;
  h.biPlanes := 1;
  h.biBitCount := bpp;
  h.biCompress := 0;
  h.biSizeImage := ( xRes * yRes * bpp ) div 8;
  h.biXPPM := 2834;
  h.biYPPM := 2834;
  h.biClrUsed := usedColors;
  h.biClrImport := usedColors;
end;

procedure color2rgb( c: byte; var RGBc: BMP_RGBQuad );
begin
  with RGBc do
  begin
    case c of
      0: begin red := 0; green := 0; blue := 0; end;
      1: begin red := 0; green := 0; blue := 160; end;
      2: begin red := 0; green := 167; blue := 0; end;
      3: begin red := 0; green := 167; blue := 160; end;
      4: begin red := 160; green := 0; blue := 0; end;
      5: begin red := 160; green := 0; blue := 160; end;
      6: begin red := 160; green := 80; blue := 40; end;
      7: begin red := 160; green := 167; blue := 160; end;
      8: begin red := 80; green := 80; blue := 80; end;
      9: begin red := 80; green := 80; blue := 255; end;
     10: begin red := 80; green := 248; blue := 80; end;
     11: begin red := 80; green := 248; blue := 255; end;
     12: begin red := 255; green := 80; blue := 80; end;
     13: begin red := 255; green := 80; blue := 255; end;
     14: begin red := 255; green := 248; blue := 80; end;
     15: begin red := 255; green := 255; blue := 255; end;
    end;
  end;
end;

procedure initColors;
var
  i: byte;
begin
  for i := 0 to 255 do
    with bmpColors[i] do
    begin
      red := 0;
      green := 0;
      blue := 0;
      reserved := 0;
    end;
  color2rgb( hintergrund, bmpColors[1] );
  color2rgb( vordergrund, bmpColors[2] );
end;

procedure karteLesen;
var
  f: text;
  i, j: byte;
  zeile: string;
begin
  assign( f, dateiName + '.vnl' );
  {$I-}
  reset( f );
  {$I+}
  if ioResult <> 0 then
  begin
    clrscr;
    writeln('Fehler beim ffnen der Datei ''', dateiName, '''!' );
    writeln;
    writeln('Info:      dieses Programm wandelt vNibbles-Level-Dateien (.vnl) in');
    writeln('           Bitmap-Dateien (.bmp) um.');
    writeln('Benutzung: ''vnl2bmp <Dateiname>''');
    writeln('           Programm und Datei mssen im gleichen Verzeichnis sein.');
    writeln;
    write('<Eingabe> drcken zu beenden');
    dateiName := '';
    readln;
  end
  else
  begin
    i := 0;
    vordergrund := black;
    hintergrund := white;
    while i < 24 do
    begin
      if not eof( f ) then readln( f, zeile ) else zeile:=' ';
      case upcase(zeile[1]) of
      'W': vordergrund := zeilenEndWert(zeile);
      'B': hintergrund := zeilenEndWert(zeile);
      '/': if zeile[2] = '/' then;
      else
        begin
          inc(i);
          zeile:= txtLaenge(zeile, 80); {Zeile auf Lnge 80 bringen}
          for j:= 1 to length(zeile) do
          case zeile[j] of {von zeile}
            char(219): begin {}
                         feld[j, (i * 2 - 1)]:= TRUE;
                         feld[j, (i * 2)]:= TRUE;
                       end;
            char(223): begin {}
                         feld[j, (i * 2 - 1)]:= TRUE;
                         feld[j, (i * 2)]:= FALSE;
                       end;
            char(220): begin {}
                         feld[j, (i * 2 - 1)]:= FALSE;
                         feld[j, (i * 2)]:= TRUE;
                       end;
            else       begin { }
                         feld[j, (i * 2 - 1)]:= FALSE;
                         feld[j, (i * 2)]:= FALSE;
                       end;
          end; {von case zeile}
        end;
      end;
    end;
    close( f );
  end;
end;

procedure colorInfo;
var
  i: byte;
begin
  for i := 0
   to maxcolors do
  begin
    with bmpColors[i] do writeln( i:4, red:4, green:4, blue:4 );
    if (i = 23) or ( (i mod 24 = 0) and (i > 24) ) then readln;
  end;
end;

procedure holDateiname;
var
  hilfS: string;
begin
  hilfS := paramstr( 0 );
  delete( hilfs, ( length( hilfs ) - 10 ), 11 );
  dateiName := hilfS + nibbserv.nurName( paramstr( 1 ) );
end;

begin
  holDateiname;
  karteLesen;
  if dateiName <> '' then
  begin
    initColors;
    initHeader;
    headerInfo;
    readln;
    bmpSchreiben;
  {  colorInfo;
    readln;}
  end;
end.